package com.ycwl.basic.ratelimiter;

import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledThreadPoolExecutor;
import java.util.concurrent.Semaphore;
import java.util.concurrent.TimeUnit;

public class FixedRateLimiter {
    private final Semaphore semaphore = new Semaphore(1);
    private final ScheduledExecutorService scheduler = new ScheduledThreadPoolExecutor(1);

    public FixedRateLimiter(int rate, TimeUnit timeUnit) {
        // 启动一个线程每0.5秒释放一个许可
        scheduler.scheduleAtFixedRate(() -> {
            synchronized (semaphore) {
                if (semaphore.availablePermits() < 1) {
                    semaphore.release(1);
                }
            }
        }, rate, rate, timeUnit);
    }

    public void acquire() throws InterruptedException {
        synchronized (semaphore) {
            semaphore.acquire();
        }
    }

    public void shutdown() {
        scheduler.shutdown();
    }
}
