package com.ycwl.basic.ratelimiter;

import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledThreadPoolExecutor;
import java.util.concurrent.Semaphore;
import java.util.concurrent.TimeUnit;


public class SlidingWindowRateLimiter {
    private final Semaphore semaphore;
    private final ScheduledExecutorService scheduler = new ScheduledThreadPoolExecutor(1);

    public SlidingWindowRateLimiter(int maxRequestsPerSecond) {
        this.semaphore = new Semaphore(maxRequestsPerSecond);
        int rate = 1000000 / maxRequestsPerSecond;
        scheduler.scheduleAtFixedRate(() -> {
            if (semaphore.availablePermits() < maxRequestsPerSecond) {
                semaphore.release(1);
            }
        }, rate, rate, TimeUnit.MICROSECONDS);
    }

    public void acquire() throws InterruptedException {
        semaphore.acquire();
    }

    public void shutdown() {
        scheduler.shutdown();
    }
}